cordova.define("de.isaco.webrtc.RTCPeerConnection", function(require, exports, module) {
var exec = require('cordova/exec'),
    cordova = require('cordova'),
    utils = require('cordova/utils'),
    log = require('./WebRTCLog');

var RTCSignalingState = {
    0: "stable",
    1: "have-local-offer",
    2: "have-remote-offer",
    3: "have-local-pranswer",
    4: "have-remote-pranswer",
    5: "closed",
};

var RTCIceConnectionState = {
    0: "new",
    1: "checking",
    2: "connected",
    3: "completed",
    4: "failed",
    5: "disconnected",
    6: "closed",
};

var RTCIceGatheringState = {
    0: "new",
    1: "gathering",
    2: "complete",
};

var RTCPeerConnection = function(configuration, constraints) {
    log.debug("RTCPeerConnection()");
    // insert object into cache for lookup in callbacks from Objective C
    this.id = utils.createUUID();
    RTCPeerConnection.__cache[this.id] = this;
    // event handlers defined in http://www.w3.org/TR/webrtc/
    this.onnegotiationneeded = null;
    this.onicecandidate = null;
    this.onsignalingstatechange = null;
    this.onaddstream = null;
    this.onremovestream = null;
    this.oniceconnectionstatechange = null;
    this.onicechange = null;
    this.onstatechange = null;
    // attributes defined in http://www.w3.org/TR/webrtc/
    this.localDescription = null;
    this.remoteDescription = null;
    this.signalingState = null;
    this.iceGatheringState = null;
    this.iceConnectionState = null;
    // private attributes
    this.__callbacks = {};
    var cbt = ["create", "desc"];  // create = offer, answer
    for (var idx = 0; idx < cbt.length; idx++) {
        this.__callbacks[cbt[idx]] = {
            success: null,
            error: null,
        };
    }
    this.__localStreams = [];
    this.__remoteStreams = [];
    this.__tDesc = null;
    this.__tType = null;
    // call Objective C
    cordova.exec(null, log.error, "WebRTC", "initPC", [this.id, configuration, constraints]);
};

RTCPeerConnection.__cache = {};

RTCPeerConnection.__notify_event = function (event) {
  log.dump("__notify_event", JSON.stringify(event));
  var pc = RTCPeerConnection.__cache[event.pcid];
  if (!pc)
    return;
  var e, s;

  if (event.name === 'negotationneeded') {
    if (pc.onnegotiationneeded !== null) {
      pc.onnegotiationneeded();
    }
  } else if (event.name === 'icecandidate') {
    if (event.hasOwnProperty('localdescription') && event.localdescription !== null) {
      pc.localDescription = new RTCSessionDescription(event.localdescription);
    }

    e = {
      currentTarget: pc
    };
    if (event.hasOwnProperty('candidate') && event.candidate !== null)
      e.candidate = new RTCIceCandidate(event.candidate);
    else
      e.candidate = null;
    if (pc.onicecandidate !== null) {
      pc.onicecandidate(e);
    }
  } else if (event.name === 'signalingstatechange') {
    pc.signalingState = RTCSignalingState[event.state];
    if (pc.onstatechange !== null) {
      pc.onstatechange();
    }
  } else if (event.name === 'addstream') {
    s = MediaStream.getOrCreate(event.sid);
    pc.__remoteStreams.push(s);
    e = {
      stream: s,
      currentTarget: pc
    };
    if (pc.onaddstream !== null) {
      pc.onaddstream(e);
    }
  } else if (event.name === 'removestream') {
    s = MediaStream.getOrCreate(event.sid);
    i = pc.__remoteStreams.indexOf(s);
    if (i != -1) {
      pc.__remoteStreams.splice(i, 1);
    }
    e = {
      stream: s,
      currentTarget: pc
    };
    if (pc.onremovestream !== null) {
      pc.onremovestream(e);
    }
  } else if (event.name === 'iceconnectionstatechange') {
    pc.iceConnectionState = RTCIceConnectionState[event.state];
    if (pc.oniceconnectionstatechange !== null) {
      pc.oniceconnectionstatechange();
    }
    // backward compatibility for JsSIP
    if (pc.onicechange !== null) {
      pc.onicechange();
    }
  } else if (event.name === 'icegatheringstatechange') {
    pc.iceGatheringState = RTCIceGatheringState[event.state];
  } else if (event.name === 'destroy') {
    delete RTCPeerConnection.__cache[event.pcid];
  }
};

RTCPeerConnection.__notify_callback = function (info) {
    log.dump("__notify_callback", JSON.stringify(info));
    var pc = RTCPeerConnection.__cache[info.pcid];
    if (!pc)
        return;
    var cb = pc.__callbacks[info.name][info.type];
    if (cb === null)
        return;
    if (info.type === "success") {
        if (info.name === 'create') {  // offer or answer
            cb(info.sdp);
        } else {  // local or remote description
            if (pc.__tDesc !== null) {
                if (pc.__tType == 'local') {
                    pc.localDescription = pc.__tDesc;
                } else {
                    pc.remoteDescription = pc.__tDesc;
                }
                pc.__tDesc = null;
                pc.__tType = null;
            }
            cb();
        }
    } else {
        cb(info.error);
        if (info.name === 'desc') {
            pc.__tDesc = null;
            pc.__tType = null;
        }
    }
};

RTCPeerConnection.prototype.createOffer = function (successCallback, failureCallback, constraints) {
  this.__callbacks.create.success = function(sdp) {
      successCallback(new RTCSessionDescription(sdp));
  };
  this.__callbacks.create.error = failureCallback;
  cordova.exec(null, failureCallback, "WebRTC", "createOffer", [this.id, constraints]);
};

RTCPeerConnection.prototype.createAnswer = function (successCallback, failureCallback, constraints) {
  this.__callbacks.create.success = function(sdp) {
      successCallback(new RTCSessionDescription(sdp));
  };
  this.__callbacks.create.error = failureCallback;
  cordova.exec(null, failureCallback, "WebRTC", "createAnswer", [this.id, constraints]);
};

RTCPeerConnection.prototype.setLocalDescription = function (description, successCallback, failureCallback) {
    this.__callbacks.desc.success = successCallback;
    this.__callbacks.desc.error = failureCallback;
    if (this.iceGatheringState === 'complete' && (this.iceConnectionState === 'connected' || this.iceConnectionState === 'completed')) {
        this.__tDesc = description;
        this.__tType = 'local';
    }
    cordova.exec(null, failureCallback, "WebRTC", "setLocalDescription", [this.id, description]);
};

RTCPeerConnection.prototype.setRemoteDescription = function (description, successCallback, failureCallback) {
    this.__callbacks.desc.success = successCallback;
    this.__callbacks.desc.error = failureCallback;
    if (this.iceGatheringState === 'complete' && (this.iceConnectionState === 'connected' || this.iceConnectionState === 'completed')) {
        this.__tDesc = description;
        this.__tType = 'remote';
    }
    cordova.exec(null, failureCallback, "WebRTC", "setRemoteDescription", [this.id, description]);
};

RTCPeerConnection.prototype.updateIce = function (configuration) {
    log.debug("updateIce() not implemented yet");
};

RTCPeerConnection.prototype.addIceCandidate = function (candidate, successCallback, failureCallback) {
    log.debug("addIceCandidate() not implemented yet");
};

/* 20140127 draft
RTCPeerConnection.prototype.getConfiguration = function () {
  
};
*/

RTCPeerConnection.prototype.getLocalStreams = function () {
    return this.__localStreams;
};

RTCPeerConnection.prototype.getRemoteStreams = function () {
    return this.__remoteStreams;
};

RTCPeerConnection.prototype.getStreamById = function (streamId) {
    var idx;
    for (idx = 0; idx < this.__localStreams.length; idx++) {
        if (this.__localStreams[idx].id === streamId) {
            return this.__localStreams[idx];
        }
    }
    for (idx = 0; idx < this.__remoteStreams.length; idx++) {
        if (this.__remoteStreams[idx].id === streamId) {
            return this.__remoteStreams[idx];
        }
    }
    return null;
};

RTCPeerConnection.prototype.addStream = function (stream, constraints) {
    this.__localStreams.push(stream);
    cordova.exec(null, log.error, "WebRTC", "addStream", [this.id, stream.id, constraints]);
};

RTCPeerConnection.prototype.removeStream = function (stream) {
    log.debug("removeStream() not implemented yet");
};

RTCPeerConnection.prototype.createDTMFSender = function(track) {
    return new RTCDTMFSender(this.id, track);
};

RTCPeerConnection.prototype.close = function () {
    cordova.exec(null, log.error, "WebRTC", "closePC", [this.id]);
};

module.exports = RTCPeerConnection;

});
